@@ -22,6 +22,7 @@ BSgenome_datapkgs <- c(
   "BSgenome.Celegans.UCSC.ce11",
   "BSgenome.Cfamiliaris.UCSC.canFam2",
   "BSgenome.Cfamiliaris.UCSC.canFam3",
+  "BSgenome.Cjacchus.UCSC.calJac3",
   "BSgenome.Dmelanogaster.UCSC.dm2",
   "BSgenome.Dmelanogaster.UCSC.dm3",
   "BSgenome.Dmelanogaster.UCSC.dm6",
@@ -39,6 +40,7 @@ BSgenome_datapkgs <- c(
   "BSgenome.Hsapiens.UCSC.hg19",
   "BSgenome.Hsapiens.NCBI.GRCh38",
   "BSgenome.Hsapiens.UCSC.hg38",
+  "BSgenome.Mdomestica.UCSC.monDom5",
   "BSgenome.Mfascicularis.NCBI.5.0",
   "BSgenome.Mfuro.UCSC.musFur1",
   "BSgenome.Mmulatta.UCSC.rheMac2",
